use clap::{App, Arg, ArgMatches};
use serde::{Deserialize, Serialize};
use serde_json::Error;
use std::string::String;
use std::{fs::File, io::Read};

#[derive(Debug, Deserialize, Serialize)]
pub struct Config {
    pub server: String,
    pub username: String,
    pub password: String,
    pub authtoken: Option<String>,
    pub sloweffect: Option<bool>,
    pub refresh: Option<u64>,
    pub limit: Option<i64>,
}

impl Config {
    pub fn new() -> Self {
        Self {
            server: String::from("https://zabbix.acme.com/api_jsonrpc.php"),
            username: String::from("bob"),
            password: String::from("password"),
            authtoken: Some(String::from("token")),
            sloweffect: Some(true),
            refresh: Some(5u64),
            limit: Some(20i64),
        }
    }

    fn feed_missing(&mut self) {
        if self.authtoken.is_none() {
            self.authtoken = Some(String::from(""));
        }
        if self.sloweffect.is_none() {
            self.sloweffect = Some(true);
        }
        if self.refresh.is_none() {
            self.refresh = Some(5u64);
        }
        if self.limit.is_none() {
            self.limit = Some(20i64);
        }
    }

    pub fn load<'a>(&mut self, configfile: &'a str) {
        let fileopen: Result<File, std::io::Error>;
        let filemeta = std::fs::metadata(configfile);
        let fileexists = match filemeta {
            Ok(_) => true,
            Err(_err) => false,
        };

        if !fileexists {
            let _a = File::create(configfile);
        }
        fileopen = File::open(configfile);

        let mut file = match fileopen {
            Ok(f) => f,
            Err(e) => {
                panic!("{}", e);
            }
        };
        let mut contents = String::new();
        file.read_to_string(&mut contents).unwrap();
        let parse: Result<Config, Error> = serde_json::from_str(contents.as_str());
        let cfg = match parse {
            Ok(cfg) => cfg,
            Err(_e) => Config::new(),
        };
        *self = cfg;
        self.feed_missing();
        self.save(&configfile);
    }

    pub fn save<'a>(&self, configfile: &'a str) {
        let file = File::create(configfile).unwrap();
        serde_json::to_writer_pretty(file, &self).unwrap();
    }
}

pub fn argparse<'a>() -> ArgMatches<'a> {
    App::new("Zabbix Launch")
        .version("1.0")
        .author("PaulBSD <paul@paulbsd.com>")
        .about("Lights up Launchpad mini using Zabbix data")
        .arg(
            Arg::with_name("config")
                .short("c")
                .long("config")
                .value_name("FILE")
                .help("Sets a custom config file")
                .takes_value(true),
        )
        .get_matches()
}
